/******************************************************************************
 *
 * System Name   : Rocinante Core Package
 * File Name     : ValidateUtil.mjs
 * Copyright     : (C) Copyright Rocinante Ein 2019
 *
 ******************************************************************************/

 /**
  * 型や値の精査を行うクラス
  */
class ValidateUtil{

  /**
   * [is_ 第二引数の型が第一引数の文字列と一致するか精査する]
   * @param  {String}  type 精査する変数の型名
   * @param  {Object}  obj  精査する変数
   * @return {Boolean}      精査した結果
   */
  static is_(type, obj){
    let clas = Object.prototype.toString.call(obj).slice(8, -1);
    return obj !== undefined && obj !== null && clas === type;
  }

  /**
   * [isS_ 指定の型の配列かを確認する関数]
   * @param  {Function}  func 精査するための関数
   * @param  {Array}     arr  精査対象の配列
   * @return {Boolean}        配列内の全ての要素を指定の関数で精査した結果
   */
  static isS_(func, arr){

    // 配列かを精査
    if( !ValidateUtil.isArray(arr) ){
      return false;
    }

    // 配列の全ての要素を精査
    for( let i = 0; i < arr.length; i++ ){

      // 数値型以外の場合はfalse
      if( !func(arr[i]) ){
        return false;
      }
    }

    return true;
  }

  /**
   * [isNumber Numberかを精査する]
   * @param  {Number}  num 精査する変数
   * @return {Boolean}     Number型の変数か
   */
  static isNumber(num){
    return ValidateUtil.is_("Number", num);
  }

  /**
   * [isNumbers Numberの配列かを精査する]
   * @param  {Array}  num 精査する変数
   * @return {Boolean}    Number型の配列か
   */
  static isNumbers(num){
    return ValidateUtil.isS_(ValidateUtil.isNumber, num);
  }

  /**
   * [isString Stringかを精査する]
   * @param  {String}  str 精査する変数
   * @return {Boolean}     精査した結果
   */
  static isString(str){
    return ValidateUtil.is_("String", str);
  }

  /**
   * [isStrings Stringの配列かを精査する]
   * @param  {Array}  str 精査する変数
   * @return {Boolean}     精査した結果
   */
  static isStrings(str){
    return ValidateUtil.isS_(ValidateUtil.isString, str);
  }

  /**
   * [isArray Arrayかを精査する]
   * @param  {Array}  arr 精査する変数
   * @return {Boolean}    精査した結果
   */
  static isArray(arr){
    return ValidateUtil.is_("Array", arr);
  }

  /**
   * [isArrays Arrayの配列かを精査する]
   * @param  {Array}  arr 精査する変数
   * @return {Boolean}    精査した結果
   */
  static isArrays(arr){
    return ValidateUtil.isS_(ValidateUtil.isArray, arr);
  }

  /**
   * [isObject Objectかを精査する]
   * @param  {Object}  obj 精査する変数
   * @return {Boolean}     精査した結果
   */
  static isObject(obj){
    return ValidateUtil.is_("Object", obj);
  }

  /**
   * [isObjects Objectの配列かを精査する]
   * @param  {Array}  obj 精査する変数
   * @return {Boolean}    精査した結果
   */
  static isObjects(obj){
    return ValidateUtil.isS_(ValidateUtil.isObject, obj);
  }

  /**
   * [isBoolean Booleanかを精査する]
   * @param  {Boolean}  bool 精査する変数
   * @return {Boolean}       精査した結果
   */
  static isBoolean(bool){
    return ValidateUtil.is_("Boolean", bool);
  }

  /**
   * [isBooleans Booleanの配列かを精査する]
   * @param  {Array}  bool 精査する変数
   * @return {Boolean}     精査した結果
   */
  static isBooleans(bool){
    return ValidateUtil.isS_(ValidateUtil.isBoolean, bool);
  }

  /**
   * [isFunction Functionかを精査する]
   * @param  {Function}  func 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isFunction(func){
    return ValidateUtil.is_("Function", func);
  }

  /**
   * [isFunctions Functionの配列かを精査する]
   * @param  {Array}  func 精査する変数
   * @return {Boolean}     精査した結果
   */
  static isFunctions(func){
    return ValidateUtil.isS_(ValidateUtil.isFunction, func);
  }

  /**
   * [isNull nullかを精査する]
   * @param  {null}  nul 精査する変数
   * @return {Boolean}   精査した結果
   */
  static isNull(nul){
    return nul === null;
  }

  /**
   * [isNulls nullの配列かを精査する]
   * @param  {[type]}  nul 精査する変数
   * @return {Boolean}     精査した結果
   */
  static isNulls(nul){
    return ValidateUtil.isS_(ValidateUtil.isNull, nul);
  }

  /**
   * [isUndefined undefinedかを精査する]
   * @param  {Undefined}  undef 精査する変数
   * @return {Boolean}          精査した結果
   */
  static isUndefined(undef){
    return typeof undef === 'undefined';
  }

  /**
   * [isUndefineds undefinedかを精査する]
   * @param  {Array}  undef 精査する変数
   * @return {Boolean}      精査した結果
   */
  static isUndefineds(undef){
    return ValidateUtil.isS_(ValidateUtil.isUndefined, undef);
  }

  /**
   * [isTrue trueかを精査する]
   * @param  {Boolean}  tru 精査する変数
   * @return {Boolean}      精査した結果
   */
  static isTrue(tru){
    return (ValidateUtil.isBoolean(tru) && (tru === true));
  }

  /**
   * [isTrues trueの配列かを精査する]
   * @param  {Array}  tru 精査する変数
   * @return {Boolean}    精査した結果
   */
  static isTrues(tru){
    return ValidateUtil.isS_(ValidateUtil.isTrue, tru);
  }

  /**
   * [isFalse falseかを精査する]
   * @param  {Boolean}  fal 精査する変数
   * @return {Boolean}      精査した結果
   */
  static isFalse(fal){
    return (ValidateUtil.isBoolean(fal) && (fal === false));
  }

  /**
   * [isFalses falseの配列かを精査する]
   * @param  {Boolean}  fal 精査する変数
   * @return {Boolean}      精査した結果
   */
  static isFalses(fal){
    return ValidateUtil.isS_(ValidateUtil.isFalse, fal);
  }

  /**
   * [isRegExp 第二引数の型が第一引数の文字列と一致するか精査する]
   * @param  {String}  str       精査する文字列
   * @param  {String}  regExpStr 正規表現文字列
   * @return {Boolean}           精査した結果
   */
  static isRegExp(str, regExpStr){
    // 正規表現文字列が文字列型でかつ、正規業源の範囲だった場合
    return ValidateUtil.isString(regExpStr) && (new RegExp(regExpStr)).test(str);
  }

  /**
   * [isPositiveNumber 正の数かを精査。]
   * @param  {Number}  number 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isPositiveNumber(number) {
    // 数字でかつ、0より大きい場合
    return (ValidateUtil.isNumber(number) && (number > 0));
  }

  /**
   * [isPositiveNumberｓ 正の数の配列かを精査。]
   * @param  {Array}  number 精査する変数
   * @return {Boolean}       精査した結果
   */
  static isPositiveNumbers(number) {
    return ValidateUtil.isS_(ValidateUtil.isPositiveNumber, number);
  }

  /**
   * [isNegativeNumber 負の数かを精査。]
   * @param  {Number}  number 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isNegativeNumber(number) {
    // 数字でかつ、0より小さい場合
    return (ValidateUtil.isNumber(number) && (number < 0));
  }

  /**
   * [isNegativeNumbers 負の数の配列かを精査。]
   * @param  {Array}  number 精査する変数
   * @return {Boolean}       精査した結果
   */
  static isNegativeNumbers(number) {
    return ValidateUtil.isS_(ValidateUtil.isNegativeNumber, number);
  }

  /**
   * [isInteger 整数かを精査。]
   * @param  {Number}  number 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isInteger(number) {
    // もし空文字または数字の正規表現の範囲外だった場合は
    return (ValidateUtil.isNumber(number) && Number.isInteger(number));
  }

  /**
   * [isIntegers 整数の配列かを精査。]
   * @param  {Array}  number  精査する変数
   * @return {Boolean}        精査した結果
   */
  static isIntegers(number) {
    return ValidateUtil.isS_(ValidateUtil.isInteger, number);
  }

  /**
   * [isPositiveInteger 正の整数かを精査。]
   * @param  {Number}  number 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isPositiveInteger(number) {
    // 整数でかつ、0より大きい場合
    return (ValidateUtil.isInteger(number) && (ValidateUtil.isPositiveNumber(number)));
  }

  /**
   * [isPositiveInteger 正の整数の配列かを精査。]
   * @param  {Array}  number 精査する変数
   * @return {Boolean}       精査した結果
   */
  static isPositiveIntegers(number) {
    return ValidateUtil.isS_(ValidateUtil.isPositiveInteger, number);
  }

  /**
   * [isNegativeInteger 負の整数かを精査。]
   * @param  {Number}  number 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isNegativeInteger(number) {
    // 整数でかつ、0より小さい場合
    return (ValidateUtil.isInteger(number) && (ValidateUtil.isNegativeNumber(number)));
  }

  /**
   * [isNegativeInteger 負の整数の配列かを精査。]
   * @param  {Array}  number 精査する変数
   * @return {Boolean}        精査した結果
   */
  static isNegativeIntegers(number) {
    return ValidateUtil.isS_(ValidateUtil.isNegativeInteger, number);
  }

  /**
   * [isUuid uuid文字列かを精査。]
   * @param  {String}  uuid UUID文字列
   * @return {Boolean}      精査した結果
   */
  static isUuid(uuid) {
    // 正規表現の範囲だった場合
    return ValidateUtil.isString(uuid) &&
           ValidateUtil.isRegExp(uuid,"[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}");
  }

  /**
   * [isUuids uuidの配列かを精査。]
   * @param  {Array}  uuid UUID文字列
   * @return {Boolean}     精査した結果
   */
  static isUuids(uuid) {
    return ValidateUtil.isS_(ValidateUtil.isUuid, uuid);
  }

  /**
   * [isBlank blankかを精査。]
   * @param  {String}  string 精査対象文字列
   * @return {Boolean}        精査した結果
   */
  static isBlank(string) {

    // 文字列が空文字、null、undefined、falseの場合はブランク
    return (ValidateUtil.isString(string) && string === "") ||
           ValidateUtil.isNull(string) ||
           ValidateUtil.isUndefined(string) ||
           ValidateUtil.isFalse(string);
  }

  /**
   * [isBlanks blankの配列かを精査。]
   * @param  {Array}  string 精査対象文字列
   * @return {Boolean}       精査した結果
   */
  static isBlanks(string) {
    return ValidateUtil.isS_(ValidateUtil.isBlank, string);
  }

  /**
   * [isDate 日付オブジェクトか精査。]
   * @param  {Date}    date 日付オブジェクト
   * @return {Boolean}      精査した結果
   */
  static isDate(date) {
    return ValidateUtil.is_("Date", date);
  }

  /**
   * [isDate 日付オブジェクトの配列か精査。]
   * @param  {Array}    date 日付オブジェクト
   * @return {Boolean}       精査した結果
   */
  static isDates(date) {
    return ValidateUtil.isS_(ValidateUtil.isDate, date);
  }

  /**
   * [isDateString 日付フォーマットがyyyy-mm-ddにあっているか精査]
   * @param  {String}  date      日付文字列
   * @param  {String}  delimiter 日付の区切り文字
   * @return {Boolean}           精査した結果
   */
  static isDateString(date, delimiter) {

    // delimiterが文字列では無い場合はデフォルトで/を設定
    if( !ValidateUtil.isString(delimiter) ){
      delimiter = "-";
    }

    // もし空文字または数字の正規表現の範囲外だった場合は論外
    if( !ValidateUtil.isString(date) || !ValidateUtil.isRegExp(date, `^\\d{4}${delimiter}\\d{2}${delimiter}\\d{2}$`) ){
      return false;
    }

    // yyyymmddで分割した日付
    let splitDate = date.split(delimiter);

    // 比較用の仮想的な日付
    let vartualDate = new Date(splitDate[0], splitDate[1]-1, splitDate[2]);

    // 分割したそれぞれの日付と比較用の仮想的な日付が等しい場合はtrueを返す
    // ※javascriptの日付型は2015-12-32等の値を入れると自動的に2016-01-01にする習性を利用したチェックです。
    if( vartualDate.getFullYear() == parseInt(splitDate[0], 10) &&
      vartualDate.getMonth()+1 == parseInt(splitDate[1], 10) &&
      vartualDate.getDate() && parseInt(splitDate[2], 10) ){
      return true;
    }

    return false;
  }

  /**
   * [isTimeString 時間フォーマットがHH:MM:SSにあっているか精査。]
   * @param  {String}  time      日時文字列
   * @param  {String}  delimiter 日付の区切り文字
   * @return {Boolean}           精査した結果
   */
  static isTimeString(time, delimiter) {

    // delimiterが文字列では無い場合はデフォルトで/を設定
    if( !ValidateUtil.isString(delimiter) ){
      delimiter = ":";
    }

    // もし空文字または数字の正規表現の範囲外だった場合は論外
    if( !ValidateUtil.isString(time) || !ValidateUtil.isRegExp(time, `\\d{2}${delimiter}\\d{2}${delimiter}\\d{2}$`) ){
      return false;
    }

    // yyyymmddHHMMで分割した日時
    let splitTime = time.split(delimiter);

    // 時間の精査を行う
    if( (parseInt(splitTime[0],10) < 24 && parseInt(splitTime[0],10) >= 0) &&
      (parseInt(splitTime[1],10) < 60 && parseInt(splitTime[1],10) >= 0) &&
      (parseInt(splitTime[2],10) < 60 && parseInt(splitTime[2],10) >= 0)){
      return true;
    }

    return false;
  }

  /**
   * [isDateTimeString 日付フォーマットがyyyy-mm-dd HH:MM:SSにあっているか精査。]
   * @param  {String}  dateTime  日時文字列
   * @param  {String}  ymdHmsDel 日付と時間の分割文字列
   * @param  {String}  ymdDel    日付の分割文字列
   * @param  {String}  hmsDel    時間の分割文字列
   * @return {Boolean}           精査した結果
   */
  static isDateTimeString(dateTime, ymdHmsDel, ymdDel, hmsDel) {

    // ymdHmsDelが文字列では無い場合はデフォルトで/を設定
    if( !ValidateUtil.isString(ymdHmsDel) ){
      ymdHmsDel = " ";
    }

    // ymdDelが文字列では無い場合はデフォルトで/を設定
    if( !ValidateUtil.isString(ymdDel) ){
      ymdDel = "-";
    }

    // hmsDelが文字列では無い場合はデフォルトで/を設定
    if( !ValidateUtil.isString(hmsDel) ){
      hmsDel = ":";
    }

    // 日と時間のｋギリ文字が一緒ならエラー
    if( ymdHmsDel === ymdDel || ymdHmsDel === hmsDel ){
      ymdHmsDel = " ";
      ymdDel = "-";
      hmsDel = ":";
    }

    // もし空文字または数字の正規表現の範囲外だった場合は論外
    if( !dateTime || !ValidateUtil.isRegExp(dateTime, `^\\d{4}${ymdDel}\\d{2}${ymdDel}\\d{2}${ymdHmsDel}\\d{2}${hmsDel}\\d{2}${hmsDel}\\d{2}$`) ){
      return false;
    }

    // 日付と時間を分割
    let splitDateTime = dateTime.split(ymdHmsDel);

    // Dateが正しく、Timeが正しい場合は正
    return ValidateUtil.isDateString(splitDateTime[0], ymdDel) && ValidateUtil.isTimeString(splitDateTime[1], hmsDel);
  }

  /**
   * [isIncluded 確認用配列の要素が全要素配列に含まれていることを確認する]
   * @param  {Array}  allArray   全要素配列
   * @param  {Array}  checkArray 確認用配列
   * @return {Boolean}           精査した結果
   */
  static isIncluded(allArray, checkArray){

    // 全要素配列変数が配列ではない場合は精査不整合
    if( !ValidateUtil.isArray(allArray) ){
      return false;
    }

    // 確認用配列変数が配列以外でかつ、未定義では無い場合
    if( !ValidateUtil.isArray(checkArray) && !ValidateUtil.isUndefined(checkArray) ){

      // 確認用配列変数を配列型にする
      checkArray = [checkArray];
    }

    // 確認側配列の要素を全チェックする
    for( let i = 0; i < checkArray.length; i++ ){

      // 全要素配列内に精査用配列の要素が含まれていない場合は精査失敗
      if( allArray.indexOf(checkArray[i]) === -1 ){
        return false;
      }
    }

    // 精査した結果問題無い場合は精査成功
    return true;
  }
}

module.exports = ValidateUtil;
